require 'creature'
require 'food'
require 'genotype'
require 'scene'
require 'monster'
require 'music'
require 'fade'

function Title()
	local title = {
		circles = {},
		lines = {},
	}

	function title:addCircle(x, y)
		table.insert(self.circles, {x = x, y = y, size = 10})
	end

	function title:addLine(i1, i2)
		table.insert(self.lines, {c1 = i1, c2 = i2})
	end

	function title:build()
		local circles = {
			{0, 140},
			{0, 70},
			{0, 0},
			{65, 110},
			{65, 35},

			{160, 0},
			{160, 70},
			{160, 140},

			{305, 00},
			{255, 35},
			{255, 110},
			{305, 140},
			{355, 115},
			{355, 35},

			{425, 0},
			{490, 0},
			{555, 0},
			{490, 70},
			{490, 140},

			{635, 0},
			{635, 70},
			{635, 140},

			{780, 0},
			{730, 35},
			{730, 110},
			{780, 140},
			{830, 115},
			{830, 35},
		}

		local lines = {
			{1, 2},
			{2, 3},
			{1, 4},
			{4, 2},
			{2, 5},
			{5, 3},

			{6, 7},
			{7, 8},

			{9, 10},
			{10, 11},
			{11, 12},
			{12, 13},
			{13, 14},
			{14, 9},

			{15, 16},
			{16, 17},
			{16, 18},
			{18, 19},

			{20, 21},
			{21, 22},

			{23, 24},
			{24, 25},
			{25, 26},
			{26, 27},
			{28, 23}
		}
		for _,c in ipairs(circles) do
			self:addCircle(c[1] + 100, c[2] + 200)
		end

		for _,l in ipairs(lines) do
			self:addLine(l[1], l[2])
		end
	end

	function title:mouseUp(mx, my, button)
		self.fadeOut = true
		scene:addEffect(FadeOut(1, newGame))
		love.audio.newSource('audio/ping_c.wav'):play()
	end

	function title:draw()
		love.graphics.setBlendMode('additive')

		local colour = {0x20, 0xff, 0x40, 0xff}
		for _,c in ipairs(self.circles) do
			love.graphics.setColor({colour[1], colour[2], colour[3], colour[4]/2})
			love.graphics.setLineWidth(10)
			love.graphics.circle('line', c.x, c.y, c.size)

			love.graphics.setColor(colour)
			love.graphics.setLineWidth(3)
			love.graphics.circle('line', c.x, c.y, c.size)
		end

		for _,l in ipairs(self.lines) do
			local c1 = self.circles[l.c1]
			local c2 = self.circles[l.c2]

			love.graphics.setColor({colour[1], colour[2], colour[3], colour[4]/2})
			love.graphics.setLineWidth(10)

			love.graphics.line(c1.x, c1.y, c2.x, c2.y)

			love.graphics.setColor(colour)
			love.graphics.setLineWidth(3)
			love.graphics.line(c1.x, c1.y, c2.x, c2.y)
		end
	end

	function title:update(dtime)
		for _,c in ipairs(self.circles) do
			local dx = math.random()
			local dy = math.random()
			c.x = c.x + dx - 0.5
			c.y = c.y + dy - 0.5
		end
	end

	title:build()
	return title
end

function menu()
	scene = Scene()
	scene:add(Title())
end

function newGame()
	music.begin()
	scene = Scene()

	scene:add(Creature(scene.w/2, scene.h/2, genotype.generate(), false))

	scene:add(FoodSpawner())
	scene:add(MonsterSpawner())

	local endGame = {
		done = false,
	}
	function endGame:update(dtime)
		if #scene:getTag('creature') == 0 and not self.done then
			scene:addEffect(FadeOut(5, menu))
			self.done = true
		end
	end
	scene:add(endGame)
end

function love.draw()
	love.graphics.setBackgroundColor(0x0, 0x0, 0x30)
	scene:draw()
end

function love.mousepressed(mx, my, button)
	scene:mouseDown(mx, my, button)
end

function love.mousereleased(mx, my, button)
	scene:mouseUp(mx, my, button)
end

function love.keyreleased(key, unicode)
	if key == 'q' or key == 'escape' then love.event.push('q') end
end

function love.update(dtime)
	scene:update(dtime)
	if music.thread then
		local msg = music.thread:receive('error')
		if msg then print(msg) end
	end
end

menu()
