require 'love.audio'
require 'love.filesystem'
require 'love.sound'
require 'love.timer'

musicthread = {}
local sequence = {1, 2, 3, 4}

local players = {}

local pitches = {
	0.9,
	1,
	1.2,
	1.5,
}

function MusicPlayer(pitch, interval, offset)

	local player = {
		time = -offset,
		interval = interval,
		ping = love.sound.newSoundData('audio/ping_c.wav'),
		position = 1,
		note = 1,
		pitch = pitch
	}

	function player:update(dtime)
		self.time = self.time+dtime
		if self.time >= self.interval then
			if self.position > #sequence then self.position = 1 end

			self.time = self.time-self.interval
			local source = love.audio.newSource(self.ping)

			local instruction = sequence[self.position]

			if instruction == 1 then
				self.note = self.note+1
				if self.note > #pitches then
					self.note = 1
				end
			elseif instruction == 2 then
				self.note = self.note-1
				if self.note < 1 then
					self.note = 1
				end
			elseif instruction == 3 then
				self.note = self.note*2
				while self.note > #pitches do
					self.note = self.note-#pitches
				end
				if self.note < 1 then self.note = 1 end
			elseif instruction == 4 then
				self.note = math.floor(#pitches/2)
			end

			source:setPitch(self.pitch * pitches[self.note])
			source:play()
			self.position = self.position+1
		end
	end

	return player
end

local speed = 0.45

table.insert(players, MusicPlayer(1, speed, 0))
table.insert(players, MusicPlayer(0.5, speed, speed/2))
table.insert(players, MusicPlayer(1.5, speed*2, 0))

local time = love.timer.getTime()
local thread = love.thread.getThread()
local playing = true
while playing do
	love.timer.sleep(5)
	local now = love.timer.getTime()
	local dtime = now - time
	time = now

	for _,p in ipairs(players) do
		p:update(dtime)
	end
	local s = thread:receive('sequence')
	if s then
		s = ''..s
		while #s < 4 do
			s = s..s
		end
		if #s % 4 then
			s = s:sub(1, #s - #s%4)
		end

		sequence = {}
		for i=1,#s do
			table.insert(sequence, s:sub(i, i)+0)
		end

	end
	if thread:receive('stop') then playing = false end
end
