function Scene()
	local scene = {
		entities = {},
		effects = {},
		w = love.graphics.getWidth(),
		h = love.graphics.getHeight()
	}

	function scene:add(entity)
		table.insert(self.entities, entity)
	end

	function scene:addEffect(effect)
		table.insert(self.effects, effect)
	end

	function scene:mouseDown(mx, my, button)
		for _,e in ipairs(self.entities) do
			if e.mouseDown then e:mouseDown(mx, my, button) end
		end
	end

	function scene:mouseUp(mx, my, button)
		for _,e in ipairs(self.entities) do
			if e.mouseUp then e:mouseUp(mx, my, button) end
		end
	end

	function scene:draw()
		for _,e in ipairs(self.entities) do
			if e.draw then e:draw() end
		end
		for _,e in ipairs(self.effects) do
			if e.draw then e:draw() end
		end
	end

	function scene:getTag(tag)
		local tagged = {}
		for _,e in ipairs(self.entities) do
			if e.tag == tag and not e.removed then
				table.insert(tagged, e)
			end
		end
		return tagged
	end

	function scene:rect()
		return {x = 0, y = 0, w = self.w, h = self.h}
	end

	function scene:remove(e)
		e.removed = true
	end

	function scene:update(dtime)
		for _,e in ipairs(self.entities) do
			if e.update then e:update(dtime) end
		end
		local ents = {}
		for _,e in ipairs(self.entities) do
			if not e.removed then table.insert(ents, e) end
		end
		for _,e in ipairs(self.effects) do
			if e.update then e:update(dtime) end
		end
		self.entities = ents
	end

	return scene
end
